 
 package Ext100;

################################################################
###                                                          ###
### SCaFoS                                                   ###
### Ecriture de fichiers selon divers criteres d'extraction  ###
###                                                          ###
################################################################
### version 1.00 du 2006/03/08                           BPH ###
################################################################


 use strict;
 use warnings;
 use Cwd;

 use Var100;
 use Lib100;
 use Ali100;
 use Gub100;
 
 use vars qw(@ISA @EXPORT);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw/ ExtractFile /;

 
################################################################


 my $bAff = '';       # texte pour affichage en mode graphique
 my $L;

 
################################################################
### GESTION DU FICHIER DES ESPECES PAR GROUPE ##################

# NOM : GetListSpecies 
# DESCRIPTION : recupere nom des especes dans .otu
# PARAMETRES : so
# RETOUR : Liste totale des especes presentes dans .otu
#          Nombre de groupes du .otu
sub GetListSpecies {   
    if ( !open(FIC, $SpecFile) ) {
        $bAff = "Can't open for reading the file \'$SpecFile\' !";
        if ( $FlagIHM == 1 ) { ViewInfo('Warning', $bAff, 'blue'); return('', 0); }
        else { die "\n\n $bAff\n"; }
        }
    
    #** parcours de .otu pour chaque ligne ***
    my $buf = "";
    my $b = "";
    my $ListSpe = "";
    my $lSpe;
    my $nbGrp = 0;
    my $bGrp = "";
    my $ListForbiden = '';
    my $FlagForbiden = 0;

    while ( $L = <FIC> ) {
        if ( $L =~ m/^#/ ) { next; }
        # separe nom du groupe des especes a prendre en compte pour la concatenation
        $lSpe = "<<<<<";
        if ( $L =~ m/(.*):(.*)/ ) {
            # recupere  le nom du groupe
            $bGrp = $1;
            # recupere la liste des especes du groupe
            $buf = $2;
####            $bGrp =~ s/^\s//g;
####            $bGrp =~ s/\s$//g;
            $bGrp =~ s/^\s+//;
            $bGrp =~ s/\s+$//;
            #verifie caracteres interdits dans noms
            if ( ForbidenChar($bGrp) == 1 ) 
                { $ListForbiden .= "   ".$bGrp."\n"; $FlagForbiden = 1; }
            $buf .= ',';
            my $n1 = 0;
            my $n2 = 0;
            my $n3 = 0;
            $n2 = index($buf, ",", $n1);
            while ( $n2 > 0 ) {
                $b = substr($buf, $n1, $n2-$n1);
                $n3 = index($b, "(", 0);
                if ( $n3 > -1 )
                    { $b = substr($b, 0, $n3); }
                $b =~ s/^\s+//;   #supprime espaces au debut
                $b =~ s/\s+$//;   #supprime espaces a la fin
                if ( length($b) > 0 ) {
                    $ListSpe .= ($b."<<<<<");
                    $lSpe .= ($b."<<<<<");
                    }
                $n1 = $n2 + 1;
                $n2 = index($buf, ",", $n1);
                }
            # sauvegarde la liste des especes par groupe
            if ( length($lSpe) > 0 ) {
                $hGrp{$bGrp} = $lSpe;
                $nbGrp++;
                }
            #verifie caracteres interdits dans noms
            $buf = $lSpe;
            $buf =~ s/<<<<<//g;
            if ( ForbidenChar($buf) == 1 ) 
                { $ListForbiden .= "   ".$bGrp."\n"; $FlagForbiden = 1; }
            }
        }
    close FIC;
    # arret si caracteres interdits
    if ( $FlagForbiden == 1 ) { AffError("Species file contains names with forbiden characters :\n".$ListForbiden); }
    if ( $NbMisGrp == -1 ) { $NbMisGrp = $nbGrp; }
    return ($ListSpe, $nbGrp);
}


### ECRITURE DE SOUS-FICHIERS DE SEQUENCES ALIGNEES ############

sub ExtractFile {
    my @lSpecies;
    
    #-- Lecture des especes retenues ---------------------------    
    $L = "\n\n=======================================================================\n";
    $L .= "\n Directory : $FileRep\n";
    if ( $FlagIHM == 1 ) { ViewDetails($L, 0); }
    else { print $L; }

    #-- recupere nom des especes dans .otu ---
    if ( $FlagIHM == 1 ) { ViewCurrent('Reading of species file', 'darkblue', $Current ); }
    my $ListSpe = "";
    ($ListSpe, $nbSpe) = GetListSpecies();
    if ( $nbSpe == 0 ) { return(-1); }
    else { $ListSpe = "<<<<<" . $ListSpe; }

    $TypeAli = SearchFileType();
    if ( ($TypeAli ne '.ali') and ($TypeAli ne '.fasta') and ($TypeAli ne '.phylip')  ) { return; }

    #-- liste les fichiers .ali ---
    if ( $FlagIHM == 1 ) { ViewCurrent('Reading of aligned files', 'darkblue', $Current ); }
    if ( !opendir(REP, $FileRep) ) { AffError("Can\'t open the directory \'$FileRep\'"); }
    my @T1 = grep m/\.ali$/i, readdir REP;   # liste tous les fichiers .ali sans tenir compte de la case
    closedir REP;
    if ( @T1 == 0 ) { AffError("None .ALI file in the directory \'$FileRep\'"); }

    #-- Ecriture des fichiers ----------------------------------
    if ( !open(LEN, "> $CefRep".$SepDir."all_files.len") )
        { AffError("Can't open for writing the file \'$CefRep"."/all_files.len\' !"); }
    print LEN "#File\tLength\tNo seq\n";
    my $L2;
    if ( $FlagIHM == 1 ) { ViewCurrent('Writing of species files in sub-directory', 'darkblue', $Current ); }
    if ( $FlagFile == 1 ) { 
        WriteFileAli($ListSpe, @T1); 
        $L2 = "AT LEAST ONE";
        }
    elsif ( $FlagAll == 1 ) { 
        WriteAll($ListSpe, @T1); 
        $L2 = "ALL";
        }
    elsif ( $FlagOnly == 1 ) { 
        WriteOnly($ListSpe, @T1); 
        $L2 = "ONLY";
        }
    close LEN;

    $L = "\n File of species : $SpecFile\n";
    $L .= "\n Files with $L2 selected species are in directory : $CefRep\n";
    $L .= "\n=======================================================================\n\n";
    if ( $FlagIHM == 1 ) { 
        ViewDetails($L, 0); 
        ViewCurrent('Process is terminated', 'red', $Current );
        return; 
        } 
    else { 
        print $L;
        exit(0); 
        }
}


# NOM : WriteOnly 
# DESCRIPTION : Ecriture de fichiers contenant uniquement les especes selectionnees
# PARAMETRES : so
# RETOUR : so
sub WriteOnly {
    my ($ListSpe, @T1) = @_;
    my $E;
    my $b;
    my $lFile = 0;   # longueur maximale des sequences
    my $bFile = '';  # contenu du fichier avec especes selectionnees
    my $SeqName = '';# nom de la sequence
    my $n;
    my $Seq;
    my $nbSeq;
    my $ListForbiden;
    my $FlagForbiden = 0;
       
    foreach my $F (sort @T1) {
        #-- ouverture des fichiers -----------------------------
        if ( !open(FIC, $FileRep.$SepDir.$F) ) {
            if ( $FlagIHM == 1 )
                { ViewInfo("Warning", "Can't open for reading the file \'$FileRep$SepDir$F\' !", "red"); }
            else    
                { print "\n\n   Can't open for reading the file \'$FileRep./$F\' !\n\n"; }
            next;
            }
        if ( !open(WRI, "> $CefRep$F") ) {
            if ( $FlagIHM == 1 )
                { ViewInfo("Warning", "Can't open for reading the file \'$CefRep./$F\' !", "red"); }
            else    
                { print "\n\n   Can't open for reading the file \'$CefRep./$F\' !\n\n"; }
            next;
            }
        if ( $TypeAli eq '.ali' ) { print WRI "#only sequences in $SpecFile selected by SCaFoS\n#\n"; }
        
        #-- initialisation des variables --
        $ListForbiden .= "$F\n";     #### 4.03c
        $nbSeq = 0;
        $lFile = 0;
        $bFile = '';
        $SeqName = '';
        $Seq = '';
        
        #-- cherche les especes du fichier .otu ----------------
        while ( $L = <FIC> ) {
            # copie des lignes de commentaire
            if ( $L =~ m/^#(.*)/ ) { $bFile .= $L; next; }
            $L = EndLine($L);
            # passe les lignes vides
            if ( $L eq '' ) { next; }
            
            if ( $L =~ m/^>(.*)/ ) {
                #-- sequence precedente a conserver ---
                if ( $SeqName ne '' )
                    { ($lFile, $bFile, $nbSeq) = ReWriteFile($SeqName, $Seq, $lFile, $bFile, $nbSeq); }
                #-- reinitialise variables ---
                $SeqName = "";
                $Seq = "";
                #-- recupere le nom ---
                $L = $1;
                $L =~ s/^\s*//;
                $L =~ s/\s*$//;
                
                #verifie caracteres interdits dans noms
                if ( ForbidenChar($L) == 1 ) { 
                    $ListForbiden .= "    ".$L."\n"; 
                    $FlagForbiden = 1;
                    }
                else {
                    $E = SearchSpecName($L, "@") . "<<<<<";
                    $b = "<<<<<" . $E;
                    if ( $ListSpe =~ m/$b/) {
                        # recupere nom de sequence
                        $SeqName = $L;
                        }
                    else
                        { $SeqName = ''; }
                    }
                }
            # recupere la sequence
            elsif ( length($SeqName) > 0 ) {
                $Seq .= $L;
                }
            }
        #-- derniere sequence a conserver ---
        if ( $SeqName ne '' )
            { ($lFile, $bFile, $nbSeq) = ReWriteFile($SeqName, $Seq, $lFile, $bFile, $nbSeq); }
        
        #-- ecriture du fichier --------------------------------
        if ( $TypeAli eq '.phylip' )
            { print WRI "$nbSeq $lFile\n"; }
        print WRI $bFile;
        close WRI;
        # deletion d'un fichier vide
        if ( $nbSeq == 0 )
            { system("rm -f $CefRep$SepDir$F"); }
        else {
            # sauvegarde nombre et longueur des sequences
            print LEN "$F\t$lFile\t$nbSeq\n";
            # renomme fichier
            if ( $TypeAli ne '.ali' ) {
                $L = $F;
                $L =~ s/\.ali//;
                if ( $^O eq 'linux' ) {  system("mv $CefRep$SepDir$F $CefRep$SepDir$L"); }
                else { system("move -f $CefRep$SepDir$F $CefRep$SepDir$L"); }
                }
            }
        close FIC;
        }
            
    # arret si caracteres interdits
    if ( $FlagForbiden == 1 )
        { AffError("Some files contain sequence names with forbiden characters :\n".$ListForbiden); }
}


sub ReWriteFile {
 my ($SeqName, $Seq, $lFile, $bFile, $nbSeq) = @_;
 my $n;
 
    $n = 0;
    $n++ while ( $Seq =~ m/\*|\?|\s|\$|\.|-|X/g );
    if ( ($n*100/length($Seq)) <= (100 - $MinLen)  ) {
        if ( $TypeAli eq '.phylip' )
            { $bFile .= "$SeqName $Seq\n"; }
        else
            { $bFile .= ">$SeqName\n$Seq\n"; }
        ++$nbSeq;
        }
    $lFile = $lFile < length($Seq) ? length($Seq) : $lFile;
    return($lFile, $bFile, $nbSeq);
}


# NOM : WriteAll
# DESCRIPTION : Copie des fichiers contenant toutes les especes selectionnees
# PARAMETRES : so
# RETOUR : so
sub WriteAll {
    my ($ListSpe, @T1) = @_;   
    my $E;       # espece courante
    my $F;       # fichier courant
    my $G;       # groupe courant
    my $ListGrp; # Liste des especes selectionnees pour le groupe courant
    my $bGrp;    # Liste des groupes deja rencontres lors de la lesture des fichiers .ali
    my $NbGrp;   # Nombre de groupes dans le fichiers courant
    my $lgFile;  # longueur 
    my $b;
    my $n;
    my $ListForbiden = '';
    my $FlagForbiden = 0;

    #** boucle sur les fichiers de sequences qlignees **********
    foreach $F (sort @T1) {
        $NbGrp = 0;
        $bGrp = "<<<<<";
        $lgFile = 0;
        if ( !open(FIC, $FileRep.$SepDir.$F) ) {
            if ( $FlagIHM == 1 ) { AffError("Can\'t open file \'$FileRep".'/'."$F\'"); }
            else { die "\n\n   Can\'t open tfile \'$FileRep".'/'."$F\'\n\n"; }
            }
        $ListForbiden .= "$F\n";     #### 4.03c
        while ( $L = <FIC> ) {
            if ( $L =~ m/^#(.*)/ ) { next; }    # saute les lignes de commentaire
            $L = EndLine($L);
            if ( $L eq '' ) { next; }            
            #-- verifie si l'espece courante doit etre selectionnee
            if ( $L =~ m/^>(.*)/ ) {            # nom de sequence / espece
                $L = $1;
                $L =~ s/^\s*//;
                $L =~ s/\s*$//;
                #verifie caracteres interdits dans noms
                if ( ForbidenChar($L) == 1 ) { 
                    $ListForbiden .= "    ".$L."\n"; 
                    $FlagForbiden = 1;
                    }
                else {
                    $E = "<<<<<" . SearchSpecName($L, "@") . "<<<<<";
                    }
                }
            #-- recherche taille de sequence la plus longue
            else {
                $ListGrp = "";
                for $G ( keys %hGrp ) {
                    $ListGrp = $hGrp{$G};
                    if ( $ListGrp =~ m/$E/ ) {
                        $b = "<<<<<" . $G . "<<<<<"; 
                        if ( !($bGrp =~ m/$b/) ) {
                            $n = 0;
                            $n++ while ( $L =~ m/\*|\?|\s|\$|\.|-|X/g );
                            if ( ($n*100/length($L)) <= (100 - $MinLen)  ) {
                                $bGrp .= $b;
                                $NbGrp++;
                                $lgFile = $lgFile < length($L) ? length($L) : $lgFile;
                                }
                            }
                        }
                    }
                }
            }
        close FIC;
        
        # toutes les especes sont presentes : copie du fichier
        if ( $NbGrp == $nbSpe ) { 
            $L = $F;
            if ( $TypeAli ne '.ali' ) { $L =~ s/\.ali//; }
            if ( $^O eq 'linux' ) { system("cp -f $FileRep"."$L $CefRep"); }
            else { 
            system("copy /Y $FileRep"."$L $CefRep"); }
            print LEN "$L\t$lgFile\t$NbGrp\n";
            }
        }
    # arret si caracteres interdits
    if ( $FlagForbiden == 1 )
        { AffError("Some files contain sequence names with forbiden characters :\n".$ListForbiden); }
}


# NOM : WriteFileAll 
# DESCRIPTION : Copie des fichiers contenant au moins une espece selectionnee
# PARAMETRES : so
# RETOUR : so
sub WriteFileAli {
    my ($ListSpe, @T1) = @_;
    my $SeqName;
    my $E;
    my $buf = "";
    my $nb = 0;
    my $n;
    my $ListForbiden = '';
    my $FlagForbiden = 0;
    my $lFile = 0;
    my $iSpe = 0;
    
    foreach my $F (sort @T1) {
        if ( $FlagVerb == 1 ) { 
            if ( ++$nb == 5 ) { 
                if ( $FlagIHM == 1 ) { ViewDetails($buf."\n", 0); }
                else { print $buf."\n"; }
                $buf = ""; $nb = 0; 
                }
            }
        if ( !open(FIC, $FileRep.$SepDir.$F) ) {
            if ( $FlagIHM == 1 ) { AffError("Can\'t open file \'$FileRep".'/'."$F\'"); }
            else { die "\n\n   Can\'t open tfile \'$FileRep".'/'."$F\'\n\n"; }
            }
        $lFile = 0;
        $iSpe = 0;
        $ListForbiden .= "$F\n";     #### 4.03c
        while ( $L = <FIC> ) {
            if ( $L =~ m/^#(.*)/ ) { next; }
            $L = EndLine($L);
            if ( $L eq '' ) { next; }            
            if ( $L =~ m/^>(.*)/ ) {
                # recupere nom de l'espece
                $L = $1;
                $L =~ s/^\s*//;
                $L =~ s/\s*$//;
                #verifie caracteres interdits dans noms
                if ( ForbidenChar($L) == 1 ) { 
                    $ListForbiden .= "    ".$L."\n"; 
                    $FlagForbiden = 1;
                    }
                else {
                    $E = "<<<<<" . SearchSpecName($L, "@") . "<<<<<";
                    }
                }
            else { 
                if ( $ListSpe =~ m/$E/ ) { 
                    $n = 0;
                    $n++ while ( $L =~ m/\*|\?|\s|\$|\.|-|X/g );
                    if ( ($n*100/length($L)) <= (100 - $MinLen)  )
                        {
                        # incremente le compteur de sequences
                        $iSpe++;
                        # determine longueur de la sequence
                        $lFile = $lFile < length($L) ? length($L) : $lFile;
                        }
                    }
                }
            }
        close FIC;
        
        if ( $iSpe > 0 ) { 
            $L = $F;
            if ( $TypeAli ne '.ali' ) { $L =~ s/\.ali//; }
            print LEN "$L\t$lFile\t$iSpe\n";
            if ( $^O eq 'linux' ) { system("cp -f $FileRep"."$L $CefRep"."$L"); }
            else { 
            system("copy /Y $FileRep"."$L $CefRep"."$L"); }
            }
        }
    # arret si caracteres interdits
    if ( $FlagForbiden == 1 )
        { SffError("Some files contain sequence names with forbiden characters :\n".$ListForbiden); }
}

1;

### fin du fichier                                           ###
################################################################
