
 package Lib100;

################################################################
###                                                          ###
### SCaFoS                                                   ###
### fonctions communes de SCaFoS                             ###
###                                                          ###
################################################################
### version 1.00 du 2006/03/31                           BPH ###
################################################################


 use strict;
 use warnings;

 use Var100;
 use Gub100;
 
 use vars qw(@ISA @EXPORT);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw/ Get_NameCef TooShort CountPosMis AffError EndLine CreateDir SearchSpecName SearchGrpSpe /;


################################################################
# NOM : Get_NameCef 
# DESCRIPTION : Determine le nom du fichier de concatenation
# PARAMETRES : so
# RETOUR : Nom du fichier
sub Get_NameCef {    
    ($CefRep) = @_;
    my ($L, $L2);
    
    if ( $CefRep eq "." )
        { return($FileRep."scafos"); }
    else {
        $L = rindex($CefRep, $SepDir);
        if ( $L == length($CefRep)-1 ) {
            $L2 = rindex($CefRep, $SepDir, $L-1);
            if ( $L2 >= 0 ) 
                { return($CefRep.substr($CefRep, $L2+1, $L-$L2-1)); }
            elsif ( $L2 == -1 )
                { return($CefRep.substr($CefRep, 0, $L)); }
            }
        elsif ( $L > -1 ) 
            { return($CefRep.substr($CefRep, $L+1, length($CefRep)-$L)); }
        else
            { return($CefRep.$CefRep); }
        }   
}        


################################################################
# NOM : TooShort
# DESCRIPTION : Verification longueur reelle sequence
# PARAMETRES : Nb max de 'non-caracteres' autorises, sequence
# RETOUR : 1 si le nombre de 'non-caracteres' dans $Seq est superieur ou egal a $MaxMiss, sinon 0
#          Nb de 'non-caracteres'
sub TooShort {
    my ($MaxMiss, $Seq) = @_;
    my $nb = 0;
    
    $nb++ while ( $Seq =~ m/\?|\*|\s|\$|\.|-|X/g );
    if ( $nb <= $MaxMiss ) { return(0, $nb); }
    else { return(1, $nb); }
}


################################################################
# NOM : CountPosMis 
# DESCRIPTION : Decompte des positions manquantes (?| |X|*|.|-|$)
# PARAMETRES : sequence
# RETOUR : Nombre de 'non-caracteres'
sub CountPosMis {
    my $n = 0;
    my ($b) = @_;
    
    $n++ while ( $b =~ m/\?|\s|\*|\.|\$|-|X/g );
    return($n);
}


################################################################
# NOM : AffError
# DESCRIPTION : Affichage un message d'erreur et ferme l'application
# PARAMETRES : Message a afficher
# RETOUR : Ferme application
sub AffError {
    my ($bAff) = @_;
    
    if ( $FlagIHM == 1 ) {
        ViewInfo('Warning' , $bAff, 'red');
        print "\n\n SCaFoS has encountered a problem with your dataset :\n   $bAff\n\n";
        exit(-1);
        }
    else
        { die "\n$bAff\n\n"; }
}


### Gestion des fins de lignes dos/unix ########################
sub EndLine {
    my ($Line) = @_;
    my $b = "";
    my ($n1, $n2);

    $n1 = index($Line, "\r");
    # \r en debut de chaine
    if ( $n1 == 0 ) {
        $n1 = (substr($Line,1,1) eq "\n" ) ? 2 : 1;
        $n2 = index(substr($Line, $n1), "\r");
        $b = substr($Line, $n1, $n2 - $n1 - 1);
    }
    # \r dans la chaine
    elsif ( $n1 != -1 ) {
        $n1 = (substr($Line,0,1) eq "\n" ) ? 1 : 0;
        $n2 = index(substr($Line, $n1), "\r");
        if ( $n2 != -1 )
            { $b = substr($Line, $n1, $n2 - $n1); }
        else
            { $b = substr($Line, $n1, length($Line) - $n1); }
    }
    # pas de \r
    else {
        $n1 = (substr($Line,0,1) eq "\n" ) ? 1 : 0;
        $n2 = index(substr($Line, $n1), "\n");
        if ( $n2 != -1 )
            { $b = substr($Line, $n1, $n2 - $n1); }
        else
            { $b = substr($Line, $n1, length($Line) - $n1); }
    }
    return $b;
}


################################################################
# NOM : CreateDir
# DESCRIPTION : creation d'un repertoire 
# PARAMETRES : nom du repertoire
# RETOUR : quitte l'application
sub CreateDir {
    my ($Dir) = @_;
    if ( !mkdir($Dir) )
        { AffError("Can\'t create the directory \'$Dir\'"); }
}


################################################################
# NOM : SearchSpecName
# DESCRIPTION : Decompose Nom d'espece et Numero d'acces
# PARAMETRES : chaine contenant les donnees, separateur
# RETOUR : Nom d'espece
sub SearchSpecName {
    my ($buf, $car) = @_;
    my $SpecName = "";        # nom de la sequence courante sans extension

    my $n = index($buf, $car);
    if ( $n == -1 )
        { $SpecName = $buf; }
    else
        { $SpecName = substr($buf, 0, $n); }
    return $SpecName;
}


### recherche la liste des identifiants des especes appartenant
### au meme groupe que l'espece courante #######################
sub SearchGrpSpe {
    my ($Id, $KeyFile, $flag) = @_;  # $Id= identifiant de la sequence courante
                                     # $KeyFile= nom du fichier .ali
                                     # $flag= 1:cherche unique nom sequence, sinon 0
    my $SpecName = "";        # nom de l'espece courante
    my $SeqName = "";         # nom unique d'identification de la sequence
    my $GrpName = "";         # nom du groupe auquel appartient la sequence

    # recherche le nom de l'espece courante
    my $S;
    for $S ( keys %{$hCorres{$KeyFile}} ) {
        if ( length($S) == 0 ) { next; }
        if ( ${$hCorres{$KeyFile}{$S}}[0] == $Id ) {
            $SpecName = SearchSpecName($S, "@");
            $SeqName = $S;
            last;
            }
        }
    if ( $flag == 1 ) 
        { return($SeqName); }
        
    # recherche le groupe auquel appartient l'espece courante
    my ($b1, $G) = ("", "");
    my @Grp = sort keys %{$hSeqFile{$KeyFile}};  
    for $G ( @Grp ) {
        if ( length($G) == 0 ) { next; }
        $b1 = "|" . @{$hSeqFile{$KeyFile}{$G}}[3];
        if ( $b1 =~ m/\|$SeqName\|/ ) {
            $GrpName = $G;
            last;
        }
    }
    
    # construit la liste de l'ordre des especes du groupe dans .puz
    my $PosList = "\|";
    for ( @{$hSeqFile{$KeyFile}{$GrpName}[5]} ) {
        $PosList .= ($hIdent{$KeyFile}{$_}."\|");
    }
    
    return ($SeqName, $GrpName, $PosList);
}


1;


### fin du fichier                                           ###
################################################################
